@@ -18,6 +18,16 @@
 
 """
 MailReceiver service definition
+
+If there's a user facing problem when processing an email, it will be
+bounced back to the sender.
+
+User facing problems could be:
+- Unknown user (missing uuid)
+- Public key not found
+
+Any other problem is a bug, which will be logged. Until the bug is
+fixed, the email will stay in there waiting.
 """
 
 import os
@@ -27,27 +37,95 @@ import json
 import email.utils
 import socket
 
-try:
-    import cchardet as chardet
-except ImportError:
-    import chardet
-
 from email import message_from_string
+from email.MIMEMultipart import MIMEMultipart
+from email.MIMEText import MIMEText
+from email.Utils import formatdate
+from email.header import decode_header
 
 from twisted.application.service import Service
-from twisted.internet import inotify, defer, task
+from twisted.internet import inotify, defer, task, reactor
 from twisted.python import filepath, log
 
-from leap.common.mail import get_email_charset
-from leap.soledad.common.document import SoledadDocument
 from leap.soledad.common.crypto import (
     EncryptionSchemes,
     ENC_JSON_KEY,
     ENC_SCHEME_KEY,
 )
-from leap.soledad.common.couch import CouchDatabase
+from leap.soledad.common.couch import CouchDatabase, CouchDocument
 from leap.keymanager import openpgp
 
+BOUNCE_TEMPLATE = """
+Delivery to the following recipient failed:
+    {0}
+
+Reasons:
+    {1}
+
+Original message:
+    {2}
+""".strip()
+
+
+from twisted.internet import protocol
+from twisted.internet.error import ProcessDone
+
+
+class BouncerSubprocessProtocol(protocol.ProcessProtocol):
+    """
+    Bouncer subprocess protocol that will feed the msg contents to be
+    bounced through stdin
+    """
+
+    def __init__(self, msg):
+        """
+        Constructor for the BouncerSubprocessProtocol
+
+        :param msg: Message to send to stdin when the process has
+                    launched
+        :type msg: str
+        """
+        self._msg = msg
+        self._outBuffer = ""
+        self._errBuffer = ""
+        self._d = None
+
+    def connectionMade(self):
+        self._d = defer.Deferred()
+
+        self.transport.write(self._msg)
+        self.transport.closeStdin()
+
+    def outReceived(self, data):
+        self._outBuffer += data
+
+    def errReceived(self, data):
+        self._errBuffer += data
+
+    def processEnded(self, reason):
+        if reason.check(ProcessDone):
+            self._d.callback(self._outBuffer)
+        else:
+            self._d.errback(reason)
+
+
+def async_check_output(args, msg):
+    """
+    Async spawn a process and return a defer to be able to check the
+    output with a callback/errback
+
+    :param args: the command to execute along with the params for it
+    :type args: list of str
+    :param msg: string that will be send to stdin of the process once
+                it's spawned
+    :type msg: str
+
+    :rtype: defer.Deferred
+    """
+    pprotocol = BouncerSubprocessProtocol(msg)
+    reactor.spawnProcess(pprotocol, args[0], args)
+    return pprotocol.d
+
 
 class MailReceiver(Service):
     """
@@ -55,8 +133,10 @@ class MailReceiver(Service):
     """
 
     INCOMING_KEY = 'incoming'
+    ERROR_DECRYPTING_KEY = "errdecr"
 
-    def __init__(self, mail_couch_url, users_cdb, directories):
+    def __init__(self, mail_couch_url, users_cdb, directories, bounce_from,
+                 bounce_subject):
         """
         Constructor
 
@@ -68,6 +148,10 @@ class MailReceiver(Service):
         :type users_cdb: ConnectedCouchDB
         :param directories: list of directories to monitor
         :type directories: list of tuples (path: str, recursive: bool)
+        :param bounce_from: Email address of the bouncer
+        :type bounce_from: str
+        :param bounce_subject: Subject line used in the bounced mail
+        :type bounce_subject: str
         """
         # Service doesn't define an __init__
         self._mail_couch_url = mail_couch_url
@@ -76,6 +160,9 @@ class MailReceiver(Service):
         self._domain = socket.gethostbyaddr(socket.gethostname())[0]
         self._processing_skipped = False
 
+        self._bounce_from = bounce_from
+        self._bounce_subject = bounce_subject
+
     def startService(self):
         """
         Starts the MailReceiver service
@@ -109,7 +196,7 @@ class MailReceiver(Service):
 
         :return: doc to sync with Soledad or None, None if something
                  went wrong.
-        :rtype: SoledadDocument
+        :rtype: CouchDocument
         """
         if pubkey is None or len(pubkey) == 0:
             log.msg("_encrypt_message: Something went wrong, here's all "
@@ -118,22 +205,18 @@ class MailReceiver(Service):
 
         # find message's encoding
         message_as_string = message.as_string()
-        encoding = get_email_charset(
-            message_as_string.decode("utf8", "replace"),
-            default=None)
-        if encoding is None:
-            result = chardet.detect(message_as_string)
-            encoding = result["encoding"]
 
-        doc = SoledadDocument(doc_id=str(pyuuid.uuid4()))
+        doc = CouchDocument(doc_id=str(pyuuid.uuid4()))
 
         # store plain text if pubkey is not available
         data = {'incoming': True, 'content': message_as_string}
         if pubkey is None or len(pubkey) == 0:
             doc.content = {
                 self.INCOMING_KEY: True,
+                self.ERROR_DECRYPTING_KEY: False,
                 ENC_SCHEME_KEY: EncryptionSchemes.NONE,
-                ENC_JSON_KEY: json.dumps(data, encoding=encoding)
+                ENC_JSON_KEY: json.dumps(data,
+                                         ensure_ascii=False)
             }
             return doc
 
@@ -143,7 +226,8 @@ class MailReceiver(Service):
             key = gpg.list_keys().pop()
             # We don't care about the actual address, so we use a
             # dummy one, we just care about the import of the pubkey
-            openpgp_key = openpgp._build_key_from_gpg("dummy@mail.com", key, pubkey)
+            openpgp_key = openpgp._build_key_from_gpg("dummy@mail.com",
+                                                      key, pubkey)
 
             # add X-Leap-Provenance header if message is not encrypted
             if message.get_content_type() != 'multipart/encrypted' and \
@@ -156,9 +240,10 @@ class MailReceiver(Service):
                 data = {'incoming': True, 'content': message.as_string()}
             doc.content = {
                 self.INCOMING_KEY: True,
+                self.ERROR_DECRYPTING_KEY: False,
                 ENC_SCHEME_KEY: EncryptionSchemes.PUBKEY,
                 ENC_JSON_KEY: str(gpg.encrypt(
-                    json.dumps(data, encoding=encoding),
+                    json.dumps(data, ensure_ascii=False),
                     openpgp_key.fingerprint,
                     symmetric=False))
             }
@@ -167,14 +252,14 @@ class MailReceiver(Service):
 
     def _export_message(self, uuid, doc):
         """
-        Given a UUID and a SoledadDocument, it saves it directly in the
+        Given a UUID and a CouchDocument, it saves it directly in the
         couchdb that serves as a backend for Soledad, in a db
         accessible to the recipient of the mail.
 
         :param uuid: the mail owner's uuid
         :type uuid: str
-        :param doc: SoledadDocument that represents the email
-        :type doc: SoledadDocument
+        :param doc: CouchDocument that represents the email
+        :type doc: CouchDocument
 
         :return: True if it's ok to remove the message, False
                  otherwise
@@ -239,6 +324,37 @@ class MailReceiver(Service):
 
         return uuid
 
+    @defer.inlineCallbacks
+    def _bounce_mail(self, orig_msg, filepath, reason):
+        """
+        Bounces the email contained in orig_msg to it's sender and
+        removes it from the queue.
+
+        :param orig_msg: Message that is going to be bounced
+        :type orig_msg: email.message.Message
+        :param filepath: Path for that message
+        :type filepath: twisted.python.filepath.FilePath
+        :param reason: Brief explanation about why it's being bounced
+        :type reason: str
+        """
+        to = orig_msg.get("From")
+
+        msg = MIMEMultipart()
+        msg['From'] = self._bounce_from
+        msg['To'] = to
+        msg['Date'] = formatdate(localtime=True)
+        msg['Subject'] = self._bounce_subject
+
+        decoded_to = " ".join([x[0] for x in decode_header(to)])
+        text = BOUNCE_TEMPLATE.format(decoded_to,
+                                      reason,
+                                      orig_msg.as_string())
+
+        msg.attach(MIMEText(text))
+
+        yield async_check_output(["/usr/sbin/sendmail", "-t"], msg.as_string())
+        yield self._conditional_remove(True, filepath)
+
     def sleep(self, secs):
         """
         Async sleep for a defer. Use this when you want to wait for
@@ -277,13 +393,13 @@ class MailReceiver(Service):
                             fullpath = os.path.join(root, fname)
                             fpath = filepath.FilePath(fullpath)
                             yield self._step_process_mail_backend(fpath)
-                        except Exception as e:
+                        except Exception:
                             log.msg("Error processing skipped mail: %r" % \
                                     (fullpath,))
                             log.err()
                     if not recursive:
                         break
-        except Exception as e:
+        except Exception:
             log.msg("Error processing skipped mail")
             log.err()
         finally:
@@ -310,6 +426,9 @@ class MailReceiver(Service):
             if uuid is None:
                 log.msg("Don't know how to deliver mail %r, skipping..." %
                         (filepath.path,))
+                bounce_reason = "Missing UUID: There was a problem " \
+                                "locating the user in our database."
+                yield self._bounce_mail(msg, filepath, bounce_reason)
                 defer.returnValue(None)
             log.msg("Mail owner: %s" % (uuid,))
 
@@ -320,6 +439,10 @@ class MailReceiver(Service):
             pubkey = yield self._users_cdb.getPubKey(uuid)
             if pubkey is None or len(pubkey) == 0:
                 log.msg("No public key, stopping the processing chain")
+                bounce_reason = "Missing PubKey: There was a problem " \
+                                "locating the user's public key in our " \
+                                "database."
+                yield self._bounce_mail(msg, filepath, bounce_reason)
                 defer.returnValue(None)
 
             log.msg("Encrypting message to %s's pubkey" % (uuid,))
@@ -351,3 +474,4 @@ class MailReceiver(Service):
         except Exception as e:
             log.msg("Something went wrong while processing {0!r}: {1!r}"
                     .format(filepath, e))
+            log.err()
